import geopandas as gpd

import geopandas as gpd
import pandas as pd

# Placeholder paths (user should replace with real local files for debugging)
grid_path = r"C:\Users\csanchez\PROJECTS\Proposals\MatrixMethod\data\vectors\grid.geojson"       # Polygon grid layer
buildings_path = r"C:\Users\csanchez\PROJECTS\Proposals\MatrixMethod\data\vectors\buildings.geojson"  # Polygon buildings layer
# Save result
output_path = r"C:\Users\csanchez\PROJECTS\Proposals\MatrixMethod\data\vectors\grid_with_coverage2.geojson"

# Updated logic to clip buildings to grid cells before computing area coverage
def calculate_coverage_geopandas(grid_path, buildings_path, output_path):
    # Load GeoDataFrames
    grid = gpd.read_file(grid_path)
    buildings = gpd.read_file(buildings_path)

    # Ensure CRS matches
    if grid.crs != buildings.crs:
        buildings = buildings.to_crs(grid.crs)

    # Compute grid cell area
    grid["cell_area"] = grid.geometry.area

    # Clip buildings to the grid geometry (this ensures we only count overlapping parts)
    clipped = gpd.overlay(buildings, grid, how="intersection")

    # Calculate area of clipped building segments
    clipped["building_area"] = clipped.geometry.area

    # Group by grid geometry (use index of grid if possible)
    # First we ensure the index from grid is preserved in overlay
    grid = grid.reset_index().rename(columns={"index": "grid_id"})
    clipped = gpd.overlay(buildings, grid, how="intersection")
    clipped["building_area"] = clipped.geometry.area

    # Now group by grid_id and sum building area
    area_per_cell = clipped.groupby("grid_id")["building_area"].sum().reset_index()
    area_per_cell.columns = ["grid_id", "total_building_area"]

    # Merge back into grid
    grid = grid.merge(area_per_cell, on="grid_id", how="left")
    grid["total_building_area"] = grid["total_building_area"].fillna(0)

    # Compute percentage
    grid["coverage_pct"] = (grid["total_building_area"] / grid["cell_area"]) * 100

    # Save to GeoJSON for inspection
    grid.to_file(output_path, driver="GeoJSON")
    return output_path

calculate_coverage_geopandas(grid_path, buildings_path, output_path)
